import click
import json
import os
import requests
import testindata
import shutil
from testindata.cli import account
from testindata.cli.cliUpload import CliUpload

from testindata.TDA import TDA
from testindata.utils import util


def _download(url, savePath):
    res = requests.get(url)
    saveDir = os.path.split(savePath)[0]
    if not os.path.exists(saveDir):
        os.makedirs(saveDir)

    with open(savePath, 'wb') as f:
        f.write(res.content)

def printVersion(ctx):
    if hasattr(testindata, "VERSION"):
        click.echo("")
        click.echo(".___________. _______       ___      ")
        click.echo("|           ||       \     /   \     ")
        click.echo("`---|  |----`|  .--.  |   /  ^  \    ")
        click.echo("    |  |     |  |  |  |  /  /_\  \   ")
        click.echo("    |  |     |  '--'  | /  _____  \  ")
        click.echo("    |__|     |_______/ /__/     \__\ ")
        click.echo("")
        click.echo(f'    TDA Version {testindata.VERSION}')
        click.echo('    more about: http://ai.testin.cn/')
        click.echo('    login: https://dataset.testin.cn/')
        click.echo("")
        ctx.exit()
    click.echo('TDA Version 0.0.1')
    ctx.exit()


@click.group()
@click.option("-ak", "--accessKey", 'access_key', default="", help="access key to Testin dataset system, see: https://dataset.testin.cn/accesskey")
@click.option("-host", "--host", 'host', default="https://dataset.testin.cn/", help="domain name that access to dataset system you would like to operate, default will be: https://dataset.testin.cn/")
@click.option('--debug/--no-debug', default=False, help="using debug mod or not, default is False")
@click.pass_context
def main(ctx, access_key, host, debug):
    """
    testin dataset system management tool
    """
    info = {
        "access_key": access_key,
        "host": host,
        "DEBUG": debug
    }

    _tda = None

    if access_key == "":
        configFile = account._config_filepath()
        if os.path.exists(configFile):
            with open(configFile, "r", encoding="utf-8") as cf:
                if cf.read() != "":
                    info = account._getConf()
                    info["DEBUG"] = debug

    if info["access_key"] != "":
        _tda = TDA(info["access_key"], info["host"])
        if info["DEBUG"]:
            _tda.Debug()

    ctx.obj = _tda


@main.command()
@click.pass_context
def version(ctx):
    """ show version """
    printVersion(ctx)
    ctx.exit()


@main.command()
@click.option("-ak", "--accessKey", 'access_key', default="", help="access key to Testin dataset system, see: https://dataset.testin.cn/accesskey")
@click.option("-h", "--host", 'host', default="https://dataset.testin.cn/", help="domain name that access to dataset system you would like to operate, default will be: https://dataset.testin.cn/")
def config(access_key, host):
    """ setting your account """
    configFile = account._config_filepath()
    if access_key == "":
        if account._check():
            click.echo("account:")
            print(account._getConf())
            exit()
    else:
        conf = {
            "access_key": access_key,
            "host": host,
        }
        with open(configFile, "w") as config:
            json.dump(conf, config)
            click.echo("config success")
            print(conf)
            exit()

@main.command()
@click.option("-ds", "--datasetId", 'ds_id', default="", help="the dataset you wolud like to download")
@click.option("-save", "--saveDir", 'save_dir', default="", help="save path for downloaded files")
@click.pass_context
def download(ctx, ds_id, save_dir):
    """ download dataset data """
    if ctx.obj == None:
        account._noLoginMessage()

    ctx.obj.SetDataset(ds_id)
    saveDir = os.path.join(save_dir, ds_id)
    if not os.path.exists(saveDir):
        os.makedirs(saveDir)

    page = 0
    limit = 100
    fileTotal = 1
    while True:
        offset = page * limit
        fileData = ctx.obj.GetData(offset, limit)
        if len(fileData["files"]) <= 0:
            break
        page += 1
        for file in fileData["files"]:
            picPath = file.path.split(ds_id)[1].strip("/")
            basename = os.path.basename(picPath)
            tmpPath = picPath.replace(basename, "").strip("/")
            fileDir = os.path.join(saveDir, tmpPath)
            if not os.path.exists(fileDir):
                os.makedirs(fileDir)
            filePath = os.path.join(fileDir, tmpPath, basename)
            if os.path.exists(filePath):
                fmd5 = util.getFileMd5(filePath)
                if fmd5 != file.md5:
                    _download(file.url, filePath)
                    if ctx.obj.debug:
                        print(f"[SAVE_FILE]file total: {fileTotal}, truncate file, redo: [{filePath}]")
                else:
                    if ctx.obj.debug:
                        print(f"[SAVE_FILE]file total: {fileTotal}, file already exist: [{filePath}]")
            else:
                _download(file.url, filePath)
                if ctx.obj.debug:
                    print(f"[SAVE_FILE]file total: {fileTotal}, file download and save: [{filePath}]")

            labelData = ctx.obj.GetFileAndLabel(fid=file.fid)
            jsonname = ".".join(basename.split(".")[:-1])
            jsonPath = os.path.join(fileDir, tmpPath, jsonname + "_label.json")
            with open(jsonPath, "w", encoding="utf-8") as jf:
                json.dump(labelData.anotations.labels, jf)
                if ctx.obj.debug:
                    print(f"[SAVE_LABEL]file total: {fileTotal}, save label data: [{jsonPath}]")

            fileTotal += 1

    if ctx.obj.debug:
        print("done!")

@main.command()
@click.pass_context
def clearcache(ctx):
    """ clean up the SDK caches """
    dir = os.path.split(account._config_filepath())[0]
    shutil.rmtree(dir)
    click.echo("all caches are cleaned up! :)")
    exit()


#从已有的数据集中加载数据到数据集管理系统
@main.command()
@click.option("-format", "--format", 'format', default="customize", help="your uploading-data format, default:voc")
@click.option("-fp", "--file-path", 'path', type=click.Path(exists=True), help="your uploading-data path")
@click.option("-fe", "--file-extension", 'fe', default=[""], help="your uploading-data file extension name")
@click.option("-afp", "--annotation-file-path", 'apath', type=click.Path(exists=True), help="your annotation file path")
@click.option("-at", "--annotation-type", 'at', default="xml", help="your annotation file content type: xml、json etc.")
@click.option("-afe", "--annotation-extension", 'afe', default=[""], help="your annotation file extension name")
@click.option("-ds", "--datasetId", 'ds_id', default="", help="the dataset you wolud like to upload")
@click.pass_context
def upload(ctx, format, path, fe, apath, at, afe, ds_id):
    """
    upload standard format data to testin dataset system
    """
    if not path:
        click.echo("you should specify where your files are!")
        click.echo("")
        click.echo("    use option -fp or --file-path to set your file path")
        click.echo("")
        ctx.exit()



    if (not apath) and (format in ["voc", "coco"]):
        click.echo("you should specify where your annotation files are, when you set format to voc or coco")
        click.echo("")
        click.echo("    use option -ds or --datasetId to set your dataset ID")
        click.echo("")
        ctx.exit()


    if not ds_id:
        click.echo("you should specify the dataset you would like to upload")
        click.echo("")
        click.echo("    use option -afp or --annotation-file-path to set your annotation file path")
        click.echo("")
        ctx.exit()

    ctx.obj.SetDataset(ds_id)


    conf = {
        "format":format,
        "path":path,
        "fe":fe,
        "apath":apath,
        "at":at,
        "afe":afe,
        "ds_id":ds_id,
    }

    up = CliUpload(ctx.obj, conf)
    up.AddFiles()

    ctx.obj.Upload()
    ctx.exit()




if __name__ == '__main__':
    main(obj={})






import os
import json

def _config_filepath():
    home = "USERPROFILE" if os.name == "nt" else "HOME"
    configDir = os.path.join(os.environ[home], ".tda/")
    if not os.path.exists(configDir):
        os.makedirs(configDir)

    return os.path.join(configDir, "tda.conf")


def _noLoginMessage():
    print(" you have not set your account yet, set your account with:\n")
    print("  tda config <access_key> [host] \n")
    print(" more about this command, use：\n")
    print("  tda config --help")
    exit()


def _check():
    configFile = _config_filepath()
    if not os.path.exists(configFile):
        _noLoginMessage()

    with open(configFile, "r") as config:
        try:
            jsonObj = json.load(config)
        except:
            _noLoginMessage()

        if "access_key" not in jsonObj.keys():
            _noLoginMessage()

        if jsonObj["access_key"] == None or jsonObj["access_key"] == "":
            _noLoginMessage()

    return True


def _getConf():
    configFile = _config_filepath()
    if _check():
        with open(configFile, "r") as config:
            return json.load(config)



import os

from testindata.dataset.metadata import MetaData
from testindata.dataset.labeldata import LabelData
from testindata.dataset.file import File


class UrlFile(File):
    def __init__(self, url, fileType):
        self.TYPE_IMAGE = 0
        self.TYPE_VIDEO = 1
        self.TYPE_AUDIO = 2
        self.TYPE_POINT_CLOUD = 3
        self.TYPE_FUSION_POINT_CLOUD = 4
        self.TYPE_POINT_CLOUD_SEMANTIC_SEGMENTATION = 5
        self.TYPE_TEXT = 6

        self.filepath = ""
        self.url = url

        self.metadata = MetaData({})
        self.labeldata = LabelData()
        self.filePrefix = ""
        self.fileType = fileType
        self.frameId = ""
        self.sensor = ""

        self.objectPath = ""
        self.osspath = self.url

        self.referId = ""
        self.filename = os.path.basename(self.url)
        self.md5 = ""
        self.filesize = 0
        self.type = ""



from testindata.TDA import TDA
from pascal_voc_tools import PascalXml
import os
import json
import xml

class CliUpload():
    def __init__(self, tda:TDA,  conf):
        self.tda = tda
        self.config = conf
        if conf['format'] not in ["voc", "coco", "customize"]:
            conf['format'] = "customize"

        self.format = conf['format']

        if not self.config['path']:
            raise Exception("you should specify where your files are!")


    def __VocAddFiles(self):
        """
        voc数据解析和AddFile
        :return:
        """

        if not self.config['apath']:
            raise Exception("you should specify where your annotation files are, when you set format to voc or coco")

        fileAndAnn = {}
        kList = {}
        if self.config['path']:
            for root, dirs, files in os.walk(self.config['path']):
                for filename in files:
                    if filename.endswith(tuple(self.config['fe'])):
                        fpath = os.path.join(root, filename)
                        k = "".join(fpath.replace(self.config['path'], "").split(".")[:-1]).lstrip("/").lstrip("\\")
                        kList[k] = fpath

        vList = {}
        if self.config['apath']:
            for root, dirs, files in os.walk(self.config['apath']):
                for filename in files:
                    if filename.endswith(tuple(self.config['afe'])):
                        annPath = os.path.join(root, filename)
                        v = "".join(annPath.replace(self.config['apath'], "").split(".")[:-1]).lstrip("/").lstrip("\\")
                        vList[v] = annPath


        for name, path in kList.items():
            fileAndAnn[kList[name]] = vList[name]


        for filePath, annPath in fileAndAnn.items():
            fileMetaData = {
                "format":self.format,
                "type":"file"
            }

            objectName = filePath.replace(self.config['path'], "").strip("/").strip("\\")
            f = self.tda.AddFile(filePath, objectName=objectName, metaData=fileMetaData)

            x = PascalXml()
            x.load(annPath)
            for obj in x.object:
                box = {
                    "x":obj.bndbox.xmin,
                    "y":obj.bndbox.ymin,
                    "width":obj.bndbox.xmax - obj.bndbox.xmin,
                    "height":obj.bndbox.ymax - obj.bndbox.ymin
                }

                attrs = {
                    "pose":obj.pose,
                    "truncated":obj.truncated,
                    "difficult":obj.difficult,
                }

                f.AddBox2D(box, label=obj.name, attrs=attrs)

            annMetaData = {
                "format":self.format,
                "type":"annotation"
            }

            annObjectName = annPath.replace(self.config['path'], "").strip("/").strip("\\")
            self.tda.AddFile(annPath, objectName=annObjectName, metaData=annMetaData)


    def __CoCoAddFiles(self):pass

    def __DefaultAddFiles(self):
        filePathList = []
        if self.config['path']:
            for root, dirs, files in os.walk(self.config['path']):
                for filename in files:
                    if filename.endswith(tuple(self.config['fe'])):
                        fpath = os.path.join(root, filename)
                        filePathList.append(fpath)

        annPathList = []
        if self.config['apath']:
            for root, dirs, files in os.walk(self.config['apath']):
                for filename in files:
                    if filename.endswith(tuple(self.config['afe'])):
                        annPath = os.path.join(root, filename)
                        annPathList.append(annPath)

        for fpath in filePathList:
            fileMetaData = {
                "type":"file"
            }
            objectName = fpath.replace(self.config['path'], "").strip("/").strip("\\")
            self.tda.AddFile(fpath, objectName=objectName, metaData=fileMetaData)

        for apath in annPathList:
            fileMetaData = {
                "type":"annotation"
            }
            objectName = apath.replace(self.config['path'], "").strip("/").strip("\\")
            self.tda.AddFile(apath, objectName=objectName, metaData=fileMetaData)

    def AddFiles(self):
        """
        :return:
        """
        if self.format == "voc":
            self.__VocAddFiles()
        elif self.format == "coco":
            self.__DefaultAddFiles()
        else:
            self.__DefaultAddFiles()







from qiniu import Auth, put_file, etag
import qiniu.config

class Qiniu():
    def __init__(self, upload_token):
        self.upload_token = upload_token


    #上传单个文件数据
    def PutObject(self, bucketName, objectName, filePath, content_type='application/octet-stream', metadata=None):
        try:
            # print(self.upload_token)
            # print(objectName)
            # print(filePath)
            # exit()
            ret, info = put_file(self.upload_token, objectName, filePath)
        except:
            raise Exception("failed in putting file")



import os, sys
import hashlib

from testindata.dataset.metadata import MetaData
from testindata.dataset.labeldata import LabelData
from testindata.utils import util


class File():
    def __init__(self, filepath, objectName, filePrefix, endpoint, fileType):
        self.TYPE_IMAGE = 0
        self.TYPE_VIDEO = 1
        self.TYPE_AUDIO = 2
        self.TYPE_POINT_CLOUD = 3
        self.TYPE_FUSION_POINT_CLOUD = 4
        self.TYPE_POINT_CLOUD_SEMANTIC_SEGMENTATION = 5
        self.TYPE_TEXT = 6

        self.metadata = MetaData({})
        self.labeldata = LabelData()
        self.filePrefix = filePrefix
        self.fileType = fileType
        self.frameId = ""
        self.sensor = ""
        if objectName == "":
            # fatherDir = os.path.abspath(os.path.dirname(filepath))
            # parentDir = os.path.abspath(os.path.dirname(os.path.dirname(filepath)))
            # lastDir = fatherDir.replace(parentDir, "").strip("/").strip("\\")
            # self.objectPath = lastDir + "/" + os.path.basename(filepath)
            # self.osspath = self.filePrefix + "/" + self.objectPath
            self.objectPath = os.path.basename(filepath)
            self.osspath = self.filePrefix + "/" + self.objectPath
        else:
            if len(objectName) > 2048:
                raise Exception("objectName length must less than 2048!")
            self.objectPath = objectName.replace("\\", "/")
            self.osspath = self.filePrefix + "/" + self.objectPath

        if "http://" in endpoint or "https://" in endpoint:
            self.osspath = endpoint.rstrip("/") + "/" + self.osspath
        else:
            self.osspath = "http://" + endpoint.strip("/") + "/" + self.osspath


        self.referId = ""
        self.filepath = filepath
        self.filename = os.path.basename(filepath)
        if len(self.filename) > 256:
            raise Exception("file basename length must less than 256!")

        self.md5 = util.getFileMd5(self.filepath)
        self.filesize = int(util.getFileSize(self.filepath))
        self.type = util.getFiletype(self.filepath)

    #数据自查
    def SelfCheck(self):
        # if not self.referId:
        #     raise Exception(f"referid must be set!")

        if self.fileType == self.TYPE_FUSION_POINT_CLOUD:
            if not self.frameId:
                raise Exception(f"3D fusion data must set frameId")

            if not self.sensor:
                raise Exception(f"3D fusion data must set sensor")


    def SetMetaData(self, madedata):
        self.metadata = MetaData(madedata)

    def SetReferId(self, ref_id):
        if ref_id == "":
            raise Exception(f"referId can not be empty string!")
        self.referId = ref_id

    def SetFreamId(self, frame_id = ""):
        self.frameId = frame_id

    def SetSensor(self, sensor = ""):
        self.sensor = sensor

    def AddBox2D(self, box2d, label="", instance="", attrs={}):
        '''
        AddBox2D
        :param box2d:
        :param label:
        :param instance:
        :param attrs:
        :return:
        '''
        # 数据格式检查
        if not type(box2d) is dict:
            raise Exception(f"box must be a dict, {type(box2d)} gavin")

        keys = list(set(box2d.keys()))
        keys.sort()
        if keys != ['height', 'width', 'x', 'y']:
            raise Exception(f"box keys must be ['height', 'width', 'x', 'y'], {keys} gavin")

        data = {
            "x":float(box2d["x"]),
            "y":float(box2d["y"]),
            "width":float(box2d["width"]),
            "height":float(box2d["height"]),
        }

        self.labeldata.AddLabels(label=label, instance=str(instance), attrs=attrs, type="box2d", data=data)

    #添加椭圆
    def AddEllipse(self, ellipse, label="", instance="", attrs={}):
        '''
        AddEllipse
        :param ellipse:
        :param label:
        :param instance:
        :param attrs:
        :return:
        '''
        if not type(ellipse) is dict:
            raise Exception(f"ellipse must be a dict, {type(ellipse)} gavin")

        keys = list(set(ellipse.keys()))
        keys.sort()
        if keys != ['height', 'width', 'x', 'y']:
            raise Exception(f"ellipse keys must be ['height', 'width', 'x', 'y'], {keys} gavin")

        data = {
            "x":float(ellipse["x"]),
            "y":float(ellipse["y"]),
            "width":float(ellipse["width"]),
            "height":float(ellipse["height"]),
        }

        self.labeldata.AddLabels(label=label, instance=str(instance), attrs=attrs, type="ellipse", data=data)


    def AddPolygon(self, polygon, label="", instance="", attrs={}, index = 0):
        '''
        AddPolygon
        :param polygon:
        :param label:
        :param instance:
        :param attrs:
        :param index:
        :return:
        '''
        if not type(polygon) is list:
            raise Exception(f"polygon must be a list, {type(polygon)} gavin")

        data = []
        for poly in polygon:
            keys = list(set(poly.keys()))
            if keys != ["x", "y"] and keys != ["y", "x"]:
                raise Exception(f"poly point keys must be ['x', 'y'], {list(set(poly.keys()))} gavin")
            data.append({"x":float(poly["x"]), "y":float(poly["y"])})

        self.labeldata.AddLabels(label=label, instance=str(instance), attrs=attrs, type="polygon", data=data, index=index)

    def AddLine(self, line, label="", instance="", attrs={}):
        '''
        AddLine
        :param line:
        :param label:
        :param instance:
        :param attrs:
        :return:
        '''
        if not type(line) is list:
            raise Exception(f"line must be a list, {type(line)} gavin")

        data = []
        for point in line:
            keys = list(set(point.keys()))
            if keys != ["x", "y"] and keys != ["y", "x"]:
                raise Exception(f"line point keys must be ['x', 'y'], {keys} gavin")
            data.append({"x": float(point["x"]), "y": float(point["y"])})

        self.labeldata.AddLabels(label=label, instance=str(instance), attrs=attrs, type="line", data=data)


    def AddCurve(self, curve, label="", instance="", attrs={}):
        '''
        AddCurve
        :param curve:
        :param label:
        :param instance:
        :param attrs:
        :return:
        '''
        if not type(curve) is list:
            raise Exception(f"curve must be a list, {type(curve)} gavin")

        data = []
        for point in curve:
            keys = list(set(point.keys()))
            if keys != ["x", "y"] and keys != ["y", "x"]:
                raise Exception(f"curve point keys must be ['x', 'y'], {keys} gavin")
            data.append({"x": float(point["x"]), "y": float(point["y"])})

        self.labeldata.AddLabels(label=label, instance=str(instance), attrs=attrs, type="curve", data=data)


    def AddPoint(self, point, label="", instance="", attrs={}):
        '''
        AddPoint
        :param point:
        :param label:
        :param instance:
        :param attrs:
        :return:
        '''
        if not type(point) is dict:
            raise Exception(f"curve must be a list, {type(point)} gavin")

        keys = list(set(point.keys()))
        if keys != ["x", "y"] and keys != ["y", "x"]:
            raise Exception(f"curve point keys must be ['x', 'y'], {keys} gavin")

        data = {"x": float(point["x"]), "y": float(point["y"])}
        self.labeldata.AddLabels(label=label, instance=str(instance), attrs=attrs, type="point", data=data)


    def AddParallel(self, parallel, label="", instance="", attrs={}):
        '''
        AddParallel
        :param parallel:
        :param label:
        :param instance:
        :param attrs:
        :return:
        '''
        if not type(parallel) is list:
            raise Exception(f"parallel must be a list, {type(parallel)} gavin")

        if len(parallel) != 4:
            raise Exception(f"parallel points numbers must be 4, {len(parallel)} gavin")

        data = []
        for point in parallel:
            keys = list(set(point.keys()))
            if keys != ["x", "y"] and keys != ["y", "x"]:
                raise Exception(f"parallel point keys must be ['x', 'y'], {keys} gavin")
            data.append({"x": float(point["x"]), "y": float(point["y"])})

        self.labeldata.AddLabels(label=label, instance=str(instance), attrs=attrs, type="parallel", data=data)


    def AddBox3D(self, box3d, label="", instance="", attrs={}):
        '''
        AddBox3D
        :param box3d:
        :param label:
        :param instance:
        :param attrs:
        :return:
        '''
        # 数据格式检查
        if not type(box3d) is dict:
            raise Exception(f"box3D must be a dict, {type(box3d)} gavin")

        keys = list(set(box3d.keys()))
        keys.sort()
        if keys != ['position', 'rotation', 'scale']:
            raise Exception(f"box3D keys must be ['position', 'rotation', 'scale'], {keys} gavin")

        data = {
            "position":{
                "x":float(box3d["position"]["x"]),
                "y":float(box3d["position"]["y"]),
                "z":float(box3d["position"]["z"]),
            },
            "rotation":{
                "x":float(box3d["rotation"]["x"]),
                "y":float(box3d["rotation"]["y"]),
                "z":float(box3d["rotation"]["z"]),
            },
            "scale":{
                "x":float(box3d["scale"]["x"]),
                "y":float(box3d["scale"]["y"]),
                "z":float(box3d["scale"]["z"]),
            }
        }

        self.labeldata.AddLabels(label=label, instance=str(instance), attrs=attrs, type="box3d", data=data)

    def AddCuboid(self, cuboid, label="", instance="", attrs={}):
        '''
        AddCuboid
        :param cuboid:
        :param label:
        :param instance:
        :param attrs:
        :return:
        '''
        # 数据格式检查
        if not type(cuboid) is dict:
            raise Exception(f"cuboid must be a dict, {type(cuboid)} gavin")

        keys = list(set(cuboid.keys()))
        keys.sort()
        if keys != ['back', 'front']:
            raise Exception(f"cuboid keys must be ['back', 'front'], {keys} gavin")

        data = {
            "back":[],
            "front":[],
        }

        if len(cuboid["back"]) != 4:
            raise Exception(f"cuboid back length must be 4, {len(cuboid['back'])} gavin")

        for point in cuboid["back"]:
            keys = list(set(point.keys()))
            if keys != ["x", "y"] and keys != ["y", "x"]:
                raise Exception(f"cuboid point keys must be ['x', 'y'], {keys} gavin")
            data["back"].append({"x": float(point["x"]), "y": float(point["y"])})

        if len(cuboid["front"]) != 4:
            raise Exception(f"cuboid front length must be 4, {len(cuboid['front'])} gavin")

        for point in cuboid["front"]:
            keys = list(set(point.keys()))
            if keys != ["x", "y"] and keys != ["y", "x"]:
                raise Exception(f"cuboid point keys must be ['x', 'y'], {keys} gavin")
            data["front"].append({"x": float(point["x"]), "y": float(point["y"])})

        self.labeldata.AddLabels(label=label, instance=str(instance), attrs=attrs, type="cuboid", data=data)

    def AddSideCuboid(self, sideCuboid, label="", instance="", attrs={}):
        '''
        AddSideCuboid
        :param sideCuboid:
        :param label:
        :param instance:
        :param attrs:
        :return:
        '''
        # 数据格式检查
        if not type(sideCuboid) is dict:
            raise Exception(f"box3D must be a dict, {type(sideCuboid)} gavin")

        keys = list(set(sideCuboid.keys()))
        keys.sort()
        if keys != ['back', 'front']:
            raise Exception(f"box3D keys must be ['back', 'front'], {keys} gavin")

        data = {
            "back":[],
            "front":[],
        }

        if len(sideCuboid["back"]) != 2:
            raise Exception(f"cuboid back length must be 4, {len(sideCuboid['back'])} gavin")

        for point in sideCuboid["back"]:
            keys = list(set(point.keys()))
            if keys != ["x", "y"] and keys != ["y", "x"]:
                raise Exception(f"cuboid point keys must be ['x', 'y'], {keys} gavin")
            data["back"].append({"x": float(point["x"]), "y": float(point["y"])})

        if len(sideCuboid["front"]) != 4:
            raise Exception(f"cuboid front length must be 4, {len(sideCuboid['front'])} gavin")

        for point in sideCuboid["front"]:
            keys = list(set(point.keys()))
            if keys != ["x", "y"] and keys != ["y", "x"]:
                raise Exception(f"cuboid point keys must be ['x', 'y'], {keys} gavin")
            data["back"].append({"x": float(point["x"]), "y": float(point["y"])})

        self.labeldata.AddLabels(label=label, instance=str(instance), attrs=attrs, type="side_cuboid", data=data)




from setuptools import setup, find_packages
import testindata

filepath = 'README.rst'

setup(
    name="testindatadev",
    version=testindata.VERSION,
    keywords=["云测", "数据集","yuncedata", "testin", "testindata"],
    description="云测数据 数据集管理平台pythonSDK",
    long_description=open(filepath, encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    license="MIT Licence",
    url = "http://ai.testin.cn/",
    author = "hide-in-code",
    author_email = "hejinlong@testin.cn",
    packages=find_packages(),
    entry_points={
      'console_scripts': [
          'tda = testindata.cli.run:main'
      ]
    },
    include_package_data=True,
    platforms="any",
    install_requires=[
        "click",
        "minio",
        "qiniu",
        "requests",
        "pyyaml"
    ],
    data_files=[filepath],
)



import sys, os
import sqlite3
import shutil

from testindata.utils import util
from testindata.dataset.file import File

class CommitDb():
    def __init__(self, commitId=""):
        self.commitId = commitId

        self.newDbFlag = False
        if self.commitId  == "":#新db
            self.commitId = "commit_" + util.getRandomSet(32)
            self.newDbFlag = True

        home = "USERPROFILE" if os.name == "nt" else "HOME"
        configDir = os.path.join(os.environ[home], ".tda/")
        if not os.path.exists(configDir):
            os.makedirs(configDir)

        self.dbDir = os.path.join(configDir, self.commitId)
        if not os.path.exists(self.dbDir):
            os.makedirs(self.dbDir)

        self.dbFile = os.path.join(self.dbDir, self.commitId + ".db")
        if not os.path.exists(self.dbFile) and not self.newDbFlag:
            raise Exception(f"commit db [{self.commitId}] not exist, have you ever commited them?")

        if not os.path.exists(self.dbFile) and self.newDbFlag:
            self.initDb()

    def initDb(self):
        self.conn = sqlite3.connect(self.dbFile)
        self.cur = self.conn.cursor()

        if self.newDbFlag:
            createSql = '''CREATE TABLE TDADATA
                   (filepath CHAR(1024) PRIMARY KEY NOT NULL,
                    osspath CHAR(1024) NOT NULL,
                    objectPath CHAR(1024) NOT NULL,
                    filename CHAR(521) NOT NULL,
                    md5 CHAR(32) NOT NULL,
                    frame_id CHAR(512) NOT NULL,
                    sensor CHAR(32) NOT NULL,
                    referid CHAR(128) NOT NULL,
                    filesize INT NOT NULL,
                    metadata CHAR(1024) NOT NULL,
                    labeldata BLOB NOT NULL
                   );'''
            self.cur.execute(createSql)

        self.conn.close()

    def close(self):
        self.conn.close()

    def getDataByFilepath(self, filepath):
        self.conn = sqlite3.connect(self.dbFile)
        self.cur = self.conn.cursor()
        try:
            res = self.cur.execute("SELECT * FROM TDADATA WHERE filepath = ?", (filepath, ))
            return res
        except:
            return None


    def insertVal(self, data = []):
        self.conn = sqlite3.connect(self.dbFile)
        self.cur = self.conn.cursor()

        for file in data:
            file.SelfCheck()
            try:
                self.cur.execute("SELECT * FROM TDADATA WHERE filepath = ?", (file.filepath,))
                res = self.cur.fetchone()
                if res != None:
                    print(file.filepath + " already commited, continue! you can updload them whenever you want.")
                    continue
                self.cur.execute("INSERT INTO TDADATA (filepath, osspath, objectPath, filename, md5, frame_id, sensor, referid, filesize, metadata, labeldata) VALUES (?,?,?,?,?,?,?,?,?,?,?)", (str(file.filepath), str(file.osspath), str(file.objectPath), str(file.filename), str(file.md5), str(file.frameId), str(file.sensor), str(file.referId), str(file.filesize), str(file.metadata.ToString()), str(file.labeldata.ToString())))
            except:
                print(f"sql: SELECT * FROM TDADATA WHERE filepath = '{file.filepath}'")
                raise Exception(f"failed in insert into tdadata, values:{str(file.filepath)}, {str(file.osspath)}, {str(file.objectPath)}, {str(file.filename)}, {str(file.md5)}, {str(file.frameId)}, {str(file.sensor)}, {str(file.referId)}, {str(file.filesize)}, {str(file.metadata.ToString())}, {str(file.labeldata.ToString())}")

        self.conn.commit()
        self.conn.close()


    def fetchAll(self):
        ret = []
        self.conn = sqlite3.connect(self.dbFile)
        self.cur = self.conn.cursor()
        sql = f"SELECT * FROM TDADATA;"
        res = self.cur.execute(sql)
        for row in res:
            tmp = {
                "filepath":row[0],
                "osspath":row[1],
                "objectPath":row[2],
                "filename":row[3],
                "md5":row[4],
                "frame_id":row[5],
                "sensor":row[6],
                "referid":row[7],
                "filesize":row[8],
                "metadata":row[9],
                "labeldata":row[10],
            }

            ret.append(tmp)
        self.conn.close()
        return ret

    def clearCache(self):
        shutil.rmtree(self.dbDir)





